local MAJOR, MINOR = "LibFail-1.0", "2"
local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

local callbacks = LibStub("CallbackHandler-1.0"):New(lib)

lib.frame = CreateFrame("Frame")

local frame = lib.frame
frame:RegisterEvent("PLAYER_ENTERING_WORLD")


local function reset()
	lib.LastTsunami = {}
	lib.ChargeCounter = {}
	lib.LastCharge = {}
	lib.ThaddiusAlive = true;
	lib.WightDeathTime = 0
	lib.LastSlime = {}
end

reset()

do
	local _, etype, f
	
	frame:SetScript("OnEvent", function (self, event, ...)
		if event == "COMBAT_LOG_EVENT_UNFILTERED" then
			_, etype = ...
			f = lib[etype]
	
			if f then
				f(self, ...)
			end
			
			return
		end

		f = lib[event]

		if f then
			f(self, ...)
		end
	end)
end


function lib:GoActive()
    frame:UnregisterEvent("RAID_ROSTER_UPDATE")
	frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self.active = true
end


function lib:PLAYER_ENTERING_WORLD(...)
	if GetNumRaidMembers() > 0 then
        self:GoActive()
	else
		if self.active then
			reset()
			self.active = nil
		end
        frame:RegisterEvent("RAID_ROSTER_UPDATE")
		frame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
end

function lib:RAID_ROSTER_UPDATE(...)
    if not self.active then
        self:GoActive()
    end
end

function lib:SPELL_DAMAGE(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool)
	if (sourceName=="Mirror Image") or (destName=="Mirror Image") then
		return
	end

	-- Lava Waves
	if (spellName=="Flame Tsunami") then
		if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
			if self.LastTsunami[destName] == nil then
				self.LastTsunami[destName] = timestamp
				
				callbacks:Fire("Fail_Sartheron_LavaWaves")
			else
				if (timestamp - lib.LastTsunami[destName]) > 10 then
					callbacks:Fire("Fail_Sartheron_LavaWaves")
				end
				self.LastTsunami[destName] = timestamp
			end
		end
	end

	-- Heigan Dance, Void Zones, Sapphiron's Breath, Frogger
	if (spellName=="Eruption") or (spellName=="Void Blast") or (spellName=="Frost Breath") or (spellName=="Explode") then
		if (sourceName:find("Fissure")) or (sourceName=="Sapphiron") or (sourceName=="Living Poison") then
			if (spellName=="Eruption") then
				self.FailType = "Heigan_Dance"
			elseif (spellName=="Void Blast") then
				self.FailType = "Any_VoidZone"
			elseif (spellName=="Frost Breath") then
				self.FailType = "Sapphiron_Breath"
			elseif (spellName=="Explode") then
				self.FailType = "Frogger"
			end
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				callbacks:Fire("Fail_"..FailType, sourceName)
			end
		end
	end

	-- Thaddius Polarity Switch
	if (self.ThaddiusAlive) then
		if (spellName=="Positive Charge") or (spellName=="Negative Charge") then
			if self.ChargeCounter[sourceName] == nil then
				self.ChargeCounter[sourceName] = 1
				self.LastCharge[sourceName] = timestamp
			elseif (timestamp - lib.LastCharge[sourceName]) < 2 then
				self.ChargeCounter[sourceName] = lib.ChargeCounter[sourceName] + 1
				self.LastCharge[sourceName] = timestamp
			else
				self.ChargeCounter[sourceName] = 1
				self.LastCharge[sourceName] = timestamp
			end

			if self.ChargeCounter[sourceName] == 3 then
				callbacks:Fire("Fail_Thaddius_PolaritySwitch", sourceName)
			end
		end
	end
end

function lib:SPELL_CAST_START(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool)
	if sourceName=="Thaddius" then
		if spellName=="Polarity Shift" then
			wipe(self.ChargeCounter)
			self.ThaddiusAlive = true;
		end
	end
end

function lib:UNIT_DIED(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags)
	if (destName=="Thaddius") then
		self.ThaddiusAlive = false;
	elseif (destName=="Stalagg") or (destName=="Feugen") then
		self.WightDeathTime = timestamp
	end
end

function lib:SPELL_AURA_APPLIED(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool)
	if (spellName=="Slime") then
		if (timestamp - self.WightDeathTime) < 120 then
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				if self.LastSlime[destName] == nil then
					callbacks:Fire("Fail_Thaddius_Jump", sourceName)
				elseif (timestamp - self.LastSlime[destName]) > 5 then
					callbacks:Fire("Fail_Thaddius_Jump", sourceName)
				end
			end
		end
	end
end

function lib:SPELL_AURA_REMOVED(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool)
	if (spellName=="Slime") then
		if (timestamp - self.WightDeathTime) < 120 then
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				self.LastSlime[destName] = timestamp
			end
		end
	end
end

